@@ -188,7 +188,7 @@ int AsfNetworkInputStream::init(const char* pszFile)
     m_iRedirectSize = 16000;
     m_iReadSize = 0;
     m_iProxyport = 0;
-    pipe(m_piPipe);
+    pipe(m_iPipeFd);
 
     avm::string::size_type start = 0, end;
 
@@ -228,8 +228,12 @@ int AsfNetworkInputStream::init(const char* pszFile)
     AVM_WRITE("ASF network reader", "server:%s filename:%s\n", m_Server.c_str(), m_Filename.c_str());
     // style hostname:port eg 192.168.10.10:8080
     char* proxyenv = getenv("HTTP_PROXY");
+    if (!proxyenv)
+	proxyenv = getenv("http_proxy");
     if (proxyenv)
     {
+	if ((strncasecmp(proxyenv, "http://", 7) == 0))
+	    proxyenv += 7;
 	m_Proxyhost = proxyenv;
 	end = m_Proxyhost.find(":");
 	if (end==avm::string::npos)
@@ -239,7 +243,7 @@ int AsfNetworkInputStream::init(const char* pszFile)
 	    avm::string p = m_Proxyhost.substr(end + 1,avm::string::npos);
 	    m_iProxyport = atoi((const char*)p);
 	}
-	m_Proxyhost = m_Proxyhost.substr(0,end);
+	m_Proxyhost = m_Proxyhost.substr(0, end);
 	m_ServerFilename += m_Filename;
 	AVM_WRITE("ASF network reader", "proxy host:%s port:%d\n", m_Proxyhost.c_str(), m_iProxyport);
     }
@@ -285,8 +289,8 @@ AsfNetworkInputStream::~AsfNetworkInputStream()
     avm::vector<NetworkIterator*>::iterator it2;
     for (it2 = m_Iterators.begin(); it2 != m_Iterators.end(); it2++)
 	(*it2)->release();
-    close(m_piPipe[0]);
-    close(m_piPipe[1]);
+    close(m_iPipeFd[0]);
+    close(m_iPipeFd[1]);
     delete[] m_pBuffer;
     delete m_pReader;
 }
@@ -314,12 +318,12 @@ bool AsfNetworkInputStream::isValid()
 
 void AsfNetworkInputStream::flushPipe()
 {
-    int pipe_flags = fcntl(m_piPipe[0], F_GETFL);
+    int pipe_flags = fcntl(m_iPipeFd[0], F_GETFL);
     char c;
-    fcntl(m_piPipe[0], F_SETFL, pipe_flags | O_NONBLOCK);
-    while (::read(m_piPipe[0], &c, 1) > 0)
+    fcntl(m_iPipeFd[0], F_SETFL, pipe_flags | O_NONBLOCK);
+    while (::read(m_iPipeFd[0], &c, 1) > 0)
 	;
-    fcntl(m_piPipe[0], F_SETFL, pipe_flags);
+    fcntl(m_iPipeFd[0], F_SETFL, pipe_flags);
 }
 
 /* time in milliseconds */
@@ -376,7 +380,7 @@ void AsfNetworkInputStream::interrupt()
     Locker locker(m_Mutex);
     m_Cond.Broadcast();
     char c = 0;
-    ::write(m_piPipe[1], &c, 1);
+    ::write(m_iPipeFd[1], &c, 1);
 }
 
 double AsfNetworkInputStream::cacheSize() const
@@ -598,18 +602,10 @@ void* AsfNetworkInputStream::threadStarter(void* arg)
 
 int AsfNetworkInputStream::createSocket()
 {
-    hostent* he;
-    int proxyport;
     avm::string sn;
+    int proxyport;
     int sp;
 
-    m_iSocket = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
-    if (m_iSocket < 0)
-    {
-	AVM_WRITE("ASF network reader", "Socket() failed ( %s )\n", strerror(errno));
-	return -1;
-    }
-
     h_errno = 0;
     if (m_iProxyport != 0)
     {
@@ -624,9 +620,8 @@ int AsfNetworkInputStream::createSocket()
 	sn = m_Server.substr(0, end);
     }
 
-    he = gethostbyname(sn.c_str());
-
     // if(!inet_aton(m_Server.c_str(), &sa.sin_addr))
+    hostent* he = gethostbyname(sn.c_str());
     if (!he || !he->h_addr_list || !he->h_addr_list[0])
     {
 	AVM_WRITE("ASF network reader", "Warning: could not resolve server name %s:%d ( %s )\n",
@@ -636,10 +631,17 @@ int AsfNetworkInputStream::createSocket()
 
     sockaddr_in sa;
     memset(&sa, 0, sizeof(sa));
-    sa.sin_family = AF_INET;
+    sa.sin_family = he->h_addrtype;
     sa.sin_port = htons(sp);
     sa.sin_addr = *(in_addr*)(he->h_addr_list[0]);
 
+    m_iSocket = socket(he->h_addrtype, SOCK_STREAM, 0);
+    if (m_iSocket < 0)
+    {
+	AVM_WRITE("ASF network reader", "Socket() failed ( %s )\n", strerror(errno));
+	return -1;
+    }
+
     int oflg = fcntl(m_iSocket, F_GETFL);
     fcntl(m_iSocket, F_SETFL, oflg | O_NONBLOCK);
     int r = connect(m_iSocket, (sockaddr*)&sa, sizeof(sa));
@@ -650,17 +652,17 @@ int AsfNetworkInputStream::createSocket()
 	{
 	    fd_set wset;
 	    fd_set rset;
-	    int mx = (m_iSocket > m_piPipe[0]) ? m_iSocket : m_piPipe[0];
+	    int mx = (m_iSocket > m_iPipeFd[0]) ? m_iSocket : m_iPipeFd[0];
 	    struct timeval tout = { 1, 0 }; // 1s timeout
 	    FD_ZERO(&wset);
 	    FD_ZERO(&rset);
 	    FD_SET(m_iSocket, &wset);
-	    FD_SET(m_piPipe[0], &rset);
+	    FD_SET(m_iPipeFd[0], &rset);
 
 	    r = select(mx + 1, &rset, &wset, 0, &tout);
 	    if (r > 0)
 	    {
-		if (FD_ISSET(m_piPipe[0], &rset))
+		if (FD_ISSET(m_iPipeFd[0], &rset))
 		{
 		    flushPipe();
 		    AVM_WRITE("ASF network reader", 1, "connect: interrupted\n");
@@ -734,10 +736,10 @@ int AsfNetworkInputStream::read(void* buffer, uint_t size)
 	fd_set fset;
 	FD_ZERO(&fset);
 	FD_SET(m_iSocket, &fset);
-	FD_SET(m_piPipe[0], &fset);
+	FD_SET(m_iPipeFd[0], &fset);
 
 	struct timeval tout = { 20, 0}; // 20s timeout
-	int mx = (m_iSocket > m_piPipe[0]) ? m_iSocket : m_piPipe[0];
+	int mx = (m_iSocket > m_iPipeFd[0]) ? m_iSocket : m_iPipeFd[0];
 
 	if (select(mx + 1, &fset, 0, 0, &tout) == 0)
 	{
@@ -746,8 +748,8 @@ int AsfNetworkInputStream::read(void* buffer, uint_t size)
 	    m_Cond.Broadcast();
 	    return -1;//continue;
 	}
-	//printf("socket %d %d\n", FD_ISSET(m_piPipe[0], &fset), FD_ISSET(m_iSocket, &fset));
-	if (FD_ISSET(m_piPipe[0], &fset))
+	//printf("socket %d %d\n", FD_ISSET(m_iPipeFd[0], &fset), FD_ISSET(m_iSocket, &fset));
+	if (FD_ISSET(m_iPipeFd[0], &fset))
 	{
 	    flushPipe();
 	    AVM_WRITE("ASF network reader", 1, "Interrupted\n");
@@ -793,22 +795,24 @@ int AsfNetworkInputStream::read(void* buffer, uint_t size)
     return -1;
 }
 
-int AsfNetworkInputStream::write(const void* buffer, uint_t size)
+int AsfNetworkInputStream::write(const void* b, uint_t size)
 {
+    const char* buffer = (const char*) b;
     int wsize = size;
     while (wsize > 0)
     {
 	int i = ::write(m_iSocket, buffer, wsize);
 	if (i <= 0)
             return i;
-	(const char*)buffer += i;
+	buffer += i;
         wsize -= i;
     }
     return size;
 }
 
-int AsfNetworkInputStream::dwrite(const void* buffer, uint_t size)
+int AsfNetworkInputStream::dwrite(const void* b, uint_t size)
 {
+    const char* buffer = (const char*) b;
     if (m_lfd < 0)
     {
 	if (m_lfd == -12345)
@@ -843,7 +847,7 @@ int AsfNetworkInputStream::dwrite(const void* buffer, uint_t size)
 	int i = ::write(m_lfd, buffer, size);
 	if (i < 0)
             return i;
-	(const char*)buffer += i;
+	buffer += i;
 	size -= i;
     }
     fsync(m_lfd);
@@ -864,13 +868,13 @@ int AsfNetworkInputStream::readContent()
 	    char c;
 	    fd_set fset;
 	    FD_ZERO(&fset);
-	    FD_SET(m_piPipe[0], &fset);
+	    FD_SET(m_iPipeFd[0], &fset);
 	    struct timeval tout = { 0, 1000000 };
-	    if (select(m_piPipe[0] + 1, &fset, 0, 0, &tout) != 0)
+	    if (select(m_iPipeFd[0] + 1, &fset, 0, 0, &tout) != 0)
 	    {
-		if (FD_ISSET(m_piPipe[0], &fset))
+		if (FD_ISSET(m_iPipeFd[0], &fset))
 		{
-		    ::read(m_piPipe[0], &c, 1);
+		    ::read(m_iPipeFd[0], &c, 1);
 		    AVM_WRITE("ASF network reader", 1, "read_content(): interrupted\n");
                     break;
         	}
@@ -1337,12 +1341,12 @@ void URLString::escape()
 	if (c && (c <= ' ' || strchr(avoid, c)))
 	{
 	    /* these should be escaped */
-	    unsigned char c1 = ((c & 0xf0) >> 4);
-	    unsigned char c2 = (c & 0x0f);
-	    if (c1 < 10) c1+='0';
-	    else c1+='A';
-	    if (c2 < 10) c2+='0';
-	    else c2+='A';
+	    unsigned char c1 = (c >> 4) & 0x0f;
+	    unsigned char c2 = c & 0x0f;
+	    if (c1 < 10) c1 += '0';
+	    else c1 += 'A';
+	    if (c2 < 10) c2 += '0';
+	    else c2 += 'A';
 	    *outbuf++ = '%';
 	    *outbuf++ = c1;
 	    *outbuf++ = c2;
