@@ -13,7 +13,6 @@ AVM_BEGIN_NAMESPACE;
 
 // reset profiler status - just for debuging
 //extern "C" void kprof_reset_stats(void);
-//template vector<AviMediaReadStream>;
 struct RiffInfoChunk {
     uint_t riff;
     const char* text;
@@ -108,6 +107,46 @@ static const struct RiffInfoChunk infos[] =
     { 0, 0 }
 };
 
+static inline void avm_get_leAVIMainHeader(AVIMainHeader* h)
+{
+    h->dwMicroSecPerFrame	= avm_get_le32(&h->dwMicroSecPerFrame);
+    h->dwMaxBytesPerSec		= avm_get_le32(&h->dwMaxBytesPerSec);
+    h->dwPaddingGranularity	= avm_get_le32(&h->dwPaddingGranularity);
+    h->dwFlags			= avm_get_le32(&h->dwFlags);
+    h->dwTotalFrames		= avm_get_le32(&h->dwTotalFrames);
+    h->dwInitialFrames		= avm_get_le32(&h->dwInitialFrames);
+    h->dwStreams		= avm_get_le32(&h->dwStreams);
+    h->dwSuggestedBufferSize	= avm_get_le32(&h->dwSuggestedBufferSize);
+    h->dwWidth			= avm_get_le32(&h->dwWidth);
+    h->dwHeight			= avm_get_le32(&h->dwHeight);
+    h->dwScale			= avm_get_le32(&h->dwScale);
+    h->dwRate			= avm_get_le32(&h->dwRate);
+    h->dwStart			= avm_get_le32(&h->dwStart);
+    h->dwLength			= avm_get_le32(&h->dwLength);
+}
+
+static inline void avm_get_leAVIStreamHeader(AVIStreamHeader* h)
+{
+    h->fccType			= avm_get_le32(&h->fccType);
+    h->fccHandler		= avm_get_le32(&h->fccHandler);
+    h->dwFlags			= avm_get_le32(&h->dwFlags);
+    h->wPriority 		= avm_get_le16(&h->wPriority);
+    h->wLanguage 		= avm_get_le16(&h->wLanguage);
+    h->dwInitialFrames		= avm_get_le32(&h->dwInitialFrames);
+    h->dwScale			= avm_get_le32(&h->dwScale);
+    h->dwRate			= avm_get_le32(&h->dwRate);
+    h->dwStart			= avm_get_le32(&h->dwStart);
+    h->dwLength			= avm_get_le32(&h->dwLength);
+    h->dwSuggestedBufferSize	= avm_get_le32(&h->dwSuggestedBufferSize);
+    h->dwQuality 		= avm_get_le32(&h->dwQuality);
+    h->dwSampleSize		= avm_get_le32(&h->dwSampleSize);
+    h->rcFrame.left		= avm_get_le16(&h->rcFrame.left);
+    h->rcFrame.right		= avm_get_le16(&h->rcFrame.right);
+    h->rcFrame.top		= avm_get_le16(&h->rcFrame.top);
+    h->rcFrame.bottom		= avm_get_le16(&h->rcFrame.bottom);
+}
+
+
 AviReadHandler::AviReadHandler(unsigned int flags)
 :m_uiFlags(flags)
 {
@@ -121,21 +160,22 @@ int AviReadHandler::init(const char* pszFile)
         return -1;
     }
 
-    if (checkDword(FOURCC_RIFF) < 0)
+    if (m_Input.readDword() != FOURCC_RIFF)
 #ifndef USE_HACK
 	return -1
 #endif
 	    ;
     /*int riff_chunk=*/
     m_Input.readDword();
-    if (checkDword(formtypeAVI) < 0)
+    if (m_Input.readDword() != formtypeAVI)
 #ifndef USE_HACK
 	return -1
 #endif
 	    ;
 
     uint_t movie_chunk = 0;
-    uint_t type, nextpos;
+    uint_t type;
+    int64_t nextpos;
     int chunk_size;
     uint32_t chunk_type;
     bool index_chunk_found = false;
@@ -164,7 +204,7 @@ int AviReadHandler::init(const char* pszFile)
             continue;
 
 	nextpos = m_Input.pos() + chunk_size + (chunk_size & 1);
-	//printf("CHUNK  %.4s  %d    %lld %d\n", (char*)&chunk_type, chunk_size, m_Input.pos(), m_Input.eof());
+	//printf("CHUNK  %.4s  %d    pos:%lld %d\n", (char*)&chunk_type, chunk_size, m_Input.pos(), m_Input.eof());
 	switch(chunk_type)
 	{
 	case FOURCC_LIST:
@@ -177,12 +217,14 @@ int AviReadHandler::init(const char* pszFile)
 	    {
                 valid_list = true;
 		type = m_Input.readDword();
+		//printf("LISTTYPE  %.4s   %lld %d\n", (char*)&type, m_Input.pos(), m_Input.eof());
 		switch (type)
 		{
 		case listtypeAVIMOVIE:
 		    movie_chunk = m_Input.pos();
 		    m_Input.seek(nextpos);
 		    // readMovieChunk();
+		    //printf("MovieChunk  %d\n", movie_chunk);
 		    break;
 		case listtypeAVIHEADER:
 		    continue;
@@ -221,18 +263,18 @@ int AviReadHandler::init(const char* pszFile)
 	m_Streams[i]->fixHeader();
 	char b[100];
         int wFormatTag;
-	if (m_Streams[i]->m_Header.fccType == streamtypeAUDIO)
+	if (m_Streams[i]->GetType() == IStream::Audio)
 	{
 	    if(m_Streams[i]->m_pcFormat && (m_Streams[i]->m_uiFormatSize >= 2))
-		wFormatTag=((WAVEFORMATEX*)(m_Streams[i]->m_pcFormat))->wFormatTag;
+		wFormatTag = ((WAVEFORMATEX*)(m_Streams[i]->m_pcFormat))->wFormatTag;
 	    else
-    		wFormatTag=m_Streams[i]->m_Header.fccHandler;
+    		wFormatTag = m_Streams[i]->m_Header.fccHandler;
 	    strncpy(b, avm_wave_format_name(wFormatTag), sizeof(b));
             b[sizeof(b) - 1] = 0;
 	}
 	else
 	{
-	    wFormatTag=m_Streams[i]->m_Header.fccHandler;
+	    wFormatTag = m_Streams[i]->m_Header.fccHandler;
 	    strncpy(b, (const char*)&m_Streams[i]->m_Header.fccHandler, 4);
             b[4] = 0;
 	}
@@ -292,26 +334,13 @@ uint_t AviReadHandler::GetHeader(void* pheader, uint_t size)
 IMediaReadStream* AviReadHandler::GetStream(uint_t stream_id,
 					    IStream::StreamType type)
 {
-    fourcc_t fcc = 0;
-
-    switch (type)
-    {
-    case IStream::Audio: fcc = streamtypeAUDIO; break;
-    case IStream::Video: fcc = streamtypeVIDEO; break;
-    default:
-	return 0;
-    }
-
     uint_t match = 0;
     avm::vector<AviReadStream*>::iterator it;
     for (it = m_Streams.begin(); it != m_Streams.end(); it++)
-	if ((*it)->m_Header.fccType == fcc)
+	if ((*it)->GetType() == type)
 	{
 	    if (match == stream_id)
-	    {
-		//AVM_WRITE("AVI reader", "GETHDR  %.4s   %.4s\n", (char*) &fccType, (char*)&it->m_Header.fccHandler);
-		return (*it); //safe for gcc3.0
-	    }
+		return (*it);
 	    match++;
 	}
     return 0;
@@ -319,59 +348,37 @@ IMediaReadStream* AviReadHandler::GetStream(uint_t stream_id,
 
 uint_t AviReadHandler::GetStreamCount(IStream::StreamType type)
 {
-    fourcc_t fcc = 0;
-
-    switch (type)
-    {
-    case IStream::Audio: fcc = streamtypeAUDIO; break;
-    case IStream::Video: fcc = streamtypeVIDEO; break;
-    default:
-	return 0;
-    }
-
     uint_t cnt = 0;
     avm::vector<AviReadStream*>::const_iterator it;
     for (it = m_Streams.begin(); it != m_Streams.end(); it++)
-	if ((*it)->m_Header.fccType == fcc)
+	if ((*it)->GetType() == type)
             cnt++;
 
     return cnt;
 }
 
-
-int AviReadHandler::checkDword(uint32_t value)
-{
-    uint_t v = m_Input.readDword();
-    if (v != value)
-    {
-	//AVM_WRITE("AVI reader", "%.4s  %.4s\n", (char*)&v, (char*)&value);
-	return -1;
-    }
-    return 0;
-}
-
 void AviReadHandler::readInfoChunk(uint_t data_size)
 {
     uint_t rs = 0;
     while (rs < data_size)
     {
-	uint32_t chunk_type = m_Input.readDword();
-	uint32_t chunk_size = m_Input.readDword();
+	uint32_t type = m_Input.readDword();
+	uint32_t size = m_Input.readDword();
 
-	if ((rs + chunk_size) < data_size)
+	if ((rs + size) < data_size)
 	{
-	    chunk_size += (chunk_size & 1);
-	    char b[chunk_size + 10];
-	    m_Input.read(b, chunk_size);
+	    size += (size & 1);
+	    char b[size + 10];
+	    m_Input.read(b, size);
 
 	    for (const RiffInfoChunk* pr = infos; pr->riff; pr++)
-		if (pr->riff == chunk_type)
+		if (pr->riff == type)
 		{
 		    AVM_WRITE("AVI reader", "InfoChunk %s:  %s\n", pr->text, b);
 		    break;
 		}
 	}
-	rs += chunk_size;
+	rs += size;
     }
 }
 
@@ -400,7 +407,7 @@ int AviReadHandler::readAVIMainHeader(uint_t data_size)
 
 int AviReadHandler::readAVIStreamHeader()
 {
-    if (checkDword(ckidSTREAMHEADER) < 0)
+    if (m_Input.readDword() != ckidSTREAMHEADER)
 	return -1;//strh
 
     AVIStreamHeader ash;
@@ -422,7 +429,7 @@ int AviReadHandler::readAVIStreamHeader()
     //if (m_Header.dwStart && m_Header.dwStart)
     //    cout << "Calc starttime " << m_Header.dwStart * (m_Header.dwRate / (double)m_Header.dwStart) << endl;
 
-    if (checkDword(ckidSTREAMFORMAT) < 0)
+    if (m_Input.readDword() != ckidSTREAMFORMAT)
 	return -1;//strf
 
     data_size = m_Input.readDword();
@@ -451,19 +458,19 @@ int AviReadHandler::readAVIStreamHeader()
 	//       s.m_Header.fccHandler, (char*)&s.m_Header.fccHandler, data_size);
 	if (bih->biCompression != ash.fccHandler)
 	{
-	    AVM_WRITE("AVI Reader", "WARNING: fccHandler differs from biCompression!\n");
+	    AVM_WRITE("AVI Reader", "Info: fccHandler differs from biCompression!\n");
 	    ash.fccHandler = bih->biCompression;
 	}
 	ash.dwSampleSize = 0;
     }
     else if (ash.fccType == streamtypeAUDIO)
     {
-	const uint_t wfs = sizeof(WAVEFORMATEX);
+	uint_t wfs = sizeof(WAVEFORMATEX) + ((WAVEFORMATEX*)format)->cbSize;
 	if (data_size < wfs)
 	{
 	    char *t = new char[wfs];
-	    memset(t, 0, wfs);
 	    memcpy(t, format, data_size);
+	    memset(t + data_size, 0, wfs - data_size);
 	    delete[] format;
 	    format = t;
 	    fsize = data_size = wfs;
@@ -548,7 +555,7 @@ int AviReadHandler::reconstructIndexChunk(uint_t offset)
     {
 	isdivx[i] = 0;
 
-	if (m_Streams[i]->m_Header.fccType == streamtypeVIDEO)
+	if (m_Streams[i]->GetType() == IStream::Video)
 	    switch (m_Streams[i]->m_Header.fccHandler)
 	    {
 	    case fccDIV3:
@@ -604,6 +611,7 @@ int AviReadHandler::reconstructIndexChunk(uint_t offset)
 	    {
                 // skip indexes
 		uint32_t size = m_Input.readDword();
+		//printf("SKEEEEEP %d\n", size);
 		m_Input.seekCur(size);
 		continue;
 	    }
@@ -623,7 +631,7 @@ int AviReadHandler::reconstructIndexChunk(uint_t offset)
 	if (clen > ((seriouslyBroken) ? MAX_CHUNK_SIZE : 0xfffffffU))
 	    continue;
 
-	//printf ("Id  %u    ps: %d  len:%u   0x%x\n", id, entry2.dwTimestamp, entry2.GetChunkLength(), ckid);
+	//printf ("Id  %u    ps: %d  len:%u   0x%x\n", id, coffset, clen, ckid);
 
 	uint_t size = clen + (clen & 1); // round
 	bool keyframe = (size > 0);
@@ -640,13 +648,20 @@ int AviReadHandler::reconstructIndexChunk(uint_t offset)
 	    case 2:
 		// DivX4 keyframe detection
 		//AVM_WRITE("AVI reader", "Dword 0x%x\n", a);
-		if (a == 0xb6010000)
+		if (a == 0xb6010000 || a == 0xb0010000)
 		{
 		    // looks like new DivX4/5 frame
-		    keyframe = ((m_Input.readByte() & 0xc0) == 0);
+                    int b = m_Input.readByte();
+		    keyframe = ((b & 0xc0) == 0);
 		    // 0x00 Key frame,  0x40 P-frame
+		    //printf("Byte 0x%x  %d", b, b);
+		    // for b001 it gives false keyframes -
+		    // parser from ffmpeg will be needed
+		    // to do a better job :(...
 		    size--;
 		}
+		//else if (a == 0xb0010000) // old OpenDivX
+		//    keyframe = true;
 		else if (a != 0x00010000) // old OpenDivX
 		    keyframe = false;
 		break;
@@ -668,6 +683,7 @@ int AviReadHandler::reconstructIndexChunk(uint_t offset)
     return 0;
 }
 
+#if 0
 AviStreamPacket* AviReadHandler::readPacket(bool fill)
 {
     int ckid = m_Input.readDword();
@@ -680,6 +696,7 @@ AviStreamPacket* AviReadHandler::readPacket(bool fill)
     }
     return 0;
 }
+#endif
 
 void AviReadHandler::PrintAVIMainHeader(const AVIMainHeader* h)
 {
