@@ -7,6 +7,8 @@
 
 #ifdef ARCH_X86
 #define FPM_INTEL
+#elif ARCH_X86_64
+#define FPM_64BIT
 #elif ARCH_POWERPC
 #define FPM_PPC
 #elif ARCH_SPARC
